<?php

namespace Drupal\rules_scheduler\Commands;

use Drush\Commands\DrushCommands;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drupal\Core\Queue\RequeueException;
use Drupal\Core\Queue\SuspendQueueException;
use Drupal\rules_scheduler\SchedulerManagerInterface;

// cspell:ignore rusch

/**
 * Drush 9+ commands for the Rules Scheduler module.
 */
class RulesSchedulerCommands extends DrushCommands {

  /**
   * The queue factory service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The queue worker manager service.
   *
   * @var \Drupal\Core\Queue\QueueWorkerManagerInterface
   */
  protected $queueWorkerManager;

  /**
   * The rules scheduler manager.
   *
   * @var \Drupal\rules_scheduler\SchedulerManagerInterface
   */
  protected $rulesSchedulerManager;

  /**
   * RulesSchedulerCommands constructor.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory service.
   * @param \Drupal\Core\Queue\QueueWorkerManagerInterface $queue_worker_manager
   *   The queue worker manager service.
   * @param \Drupal\rules_scheduler\SchedulerManagerInterface $rules_scheduler_manager
   *   The rules scheduler manager.
   */
  public function __construct(QueueFactory $queue_factory, QueueWorkerManagerInterface $queue_worker_manager, SchedulerManagerInterface $rules_scheduler_manager) {
    parent::__construct();
    $this->queueFactory = $queue_factory;
    $this->queueWorkerManager = $queue_worker_manager;
    $this->rulesSchedulerManager = $rules_scheduler_manager;
  }

  /**
   * Command callback for processing the 'rules_scheduler_tasks' queue.
   *
   * Checks for scheduled tasks to be added to the queue.
   *
   * @param array $options
   *   An associative array of options whose values come from cli, aliases,
   *   config, etc.
   *
   * @command rules:scheduler-tasks
   * @aliases rusch,rules-scheduler-tasks
   *
   * @option claim
   *   Optionally claim tasks from the queue to work on. Any value set will
   *   override the default time spent on this queue.
   *
   * @usage drush rules:scheduler-tasks
   *   Add scheduled tasks to the queue.
   * @usage drush rules:scheduler-tasks --claim
   *   Add scheduled tasks to the queue and claim items for the default amount
   *   of time.
   * @usage drush rules:scheduler-tasks --claim=30
   *   Add scheduled tasks to the queue and claim items for 30 seconds.
   *
   * @validate-module-enabled rules,rules_scheduler
   *
   * @see rules_scheduler_cron()
   * @see \Drupal\Core\Cron::run()
   * @see \Drupal\rules_scheduler\Plugin\QueueWorker\TaskWorker
   */
  public function schedulerTasks(array $options = ['claim' => FALSE]) {

    // Force Rules Scheduler to queue any ready-to-execute tasks.
    // This is exactly what Rules Scheduler does on a cron run, but
    // note this is being done OUTSIDE of hook_cron() - cron doesn't
    // trigger this, drush does.
    $this->rulesSchedulerManager->queueTasks();

    // Much of this code is taken from \Drupal\Core\Cron::processQueues().
    // It would be nice to avoid this duplication.
    $claim = $options['claim'];
    if ($claim) {
      // Fetch the queue information and worker plugin definition.
      $queue_name = 'rules_scheduler_tasks';
      $info = $this->queueWorkerManager->getDefinition($queue_name);

      // The drush option can override the default process time.
      $default_time = isset($info['cron']['time']) ? isset($info['cron']['time']) : 15;
      $lease_time = is_numeric($claim) ? (int) $claim : $default_time;
      $end = time() + $lease_time;

      // Claim items and process the queue.
      $queue = $this->queueFactory->get($queue_name);
      $queue_worker = $this->queueWorkerManager->createInstance('rules_scheduler_tasks');

      $claimed = 0;
      while (time() < $end && ($item = $queue->claimItem($lease_time))) {
        try {
          $queue_worker->processItem($item->data);
          $queue->deleteItem($item);
          $claimed++;
        }
        catch (RequeueException $e) {
          // The worker requested the task be immediately requeued.
          $queue->releaseItem($item);
        }
        catch (SuspendQueueException $e) {
          // If the worker indicates there is a problem with the whole queue,
          // release the item and skip to the next queue.
          $queue->releaseItem($item);
          watchdog_exception('cron', $e);
          // Skip processing this queue.
          continue;
        }
        catch (\Exception $e) {
          // In case of any other kind of exception, log it and leave the item
          // in the queue to be processed again later.
          watchdog_exception('cron', $e);
        }
      }

      if ($claimed) {
        $this->logger()->info(dt('Claimed and worked on @claimed scheduled tasks for up to @time seconds.', ['@claimed' => $claimed, '@time' => $lease_time]), 'success');
      }
    }
  }

}
